module ActiveAttack
  class Matrix < ApplicationRecord
    belongs_to :bundle, :class_name => 'ActiveStix::Bundle', optional: true

    has_many :reference_items, class_name: "ActiveStix::ReferenceItem", as: :referrer
    has_many :external_references, class_name: "ActiveStix::ExternalReference", through: :reference_items
    has_many :tactics, primary_key: 'stix_id', foreign_key: 'matrix_ref'

    def ingest_bundle(path)
      bundle = ActiveStix::Bundle.ingest(path)
      self.bundle = bundle
      save
      bundle
    end


    def self.ingest_json(obj)
      xmx = find_or_create_by(stix_id:obj['id'], name:obj['name'])

      if obj.has_key?('description')
        xmx.description = obj['description']
      end

      if obj.has_key?('created_by_ref')
        xmx.created_by_ref = obj['created_by_ref']
      end

      if obj.has_key?('external_references')
        obj['external_references'].each do |er|
          external_reference = ActiveStix::ExternalReference.ingest_json(er, obj['id'])
          xmx.external_references << external_reference unless ActiveStix::ReferenceItem.find_by(external_reference_id:external_reference.id, referrer_id:xmx.id, referrer_type: "ActiveAttack::Matrix")
        end
      end

      if obj.has_key?('object_marking_refs')
        # todo
      end

      if obj.has_key?('tactic_refs')
        obj['tactic_refs'].each do |tr|
          tactic_ref = ActiveAttack::Tactic.find_or_create_by(stix_id: tr)
          xmx.tactics << tactic_ref
        end
      end

      xmx.save
      xmx
    end

    def convert_to_json
      external_refs_arr = []
      external_references.each do | x |
        external_refs_arr << x.convert_to_json
      end

      marking_def_arr = []
      marking_definitions.each do | x |
        marking_def_arr << x.convert_to_json
      end


      tactic_refs_arr = []
      x_mitre_tactic_refs.each do | x |
        tactic_refs_arr << x.convert_to_json
      end

      {
          :external_references => external_refs_arr,
          :object_marking_refs => marking_def_arr,
          :id => stix_id,
          :name => name,
          :created => created_at.to_s,
          :modified => updated_at.to_s,
          :type => "x-mitre-matrix",
          :created_by_ref => created_by_ref,
          :description => description,
          :tactic_refs => tactic_refs_arr
      }
    end
  end
end
